@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import hashlib
 import os
 import shutil
@@ -7,11 +5,10 @@ import sys
 import tempfile
 import zipfile
 
-import six
 from humanize import naturalsize
 from tqdm import tqdm
 from twisted.internet import reactor
-from twisted.internet.defer import inlineCallbacks, returnValue
+from twisted.internet.defer import inlineCallbacks
 from twisted.python import log
 from wormhole import __version__, create, input_with_completion
 
@@ -84,6 +81,8 @@ class Receiver:
             self._reactor,
             tor=self._tor,
             timing=self.args.timing)
+        if self.args.debug_state:
+            w.debug_set_trace("recv", which=" ".join(self.args.debug_state), file=self.args.stdout)
         self._w = w  # so tests can wait on events too
 
         # I wanted to do this instead:
@@ -103,7 +102,7 @@ class Receiver:
         @inlineCallbacks
         def _good(res):
             yield w.close()  # wait for ack
-            returnValue(res)
+            return res
 
         # if we raise an error, we should close and then return the original
         # error (the close might give us an error, but it isn't as important
@@ -114,7 +113,7 @@ class Receiver:
                 yield w.close()  # might be an error too
             except Exception:
                 pass
-            returnValue(f)
+            return f
 
         d.addCallbacks(_good, _bad)
         yield d
@@ -173,7 +172,6 @@ class Receiver:
 
         while True:
             them_d = yield self._get_data(w)
-            # print("GOT", them_d)
             recognized = False
             if u"transit" in them_d:
                 recognized = True
@@ -188,7 +186,7 @@ class Receiver:
                 except RespondError as r:
                     self._send_data({"error": r.response}, w)
                     raise TransferError(r.response)
-                returnValue(None)
+                return None
             if not recognized:
                 log.msg("unrecognized message %r" % (them_d, ))
 
@@ -203,7 +201,7 @@ class Receiver:
         them_d = bytes_to_dict(them_bytes)
         if "error" in them_d:
             raise TransferError(them_d["error"])
-        returnValue(them_d)
+        return them_d
 
     @inlineCallbacks
     def _handle_code(self, w):
@@ -214,13 +212,21 @@ class Receiver:
         if code:
             w.set_code(code)
         else:
-            prompt = "Enter receive wormhole code: "
-            used_completion = yield input_with_completion(
-                prompt, w.input_code(), self._reactor)
-            if not used_completion:
-                print(
-                    " (note: you can use <Tab> to complete words)",
-                    file=self.args.stderr)
+            if self.args.allocate:
+                w.allocate_code(self.args.code_length)
+                code = yield w.get_code()
+                print(u"Allocated code: {}".format(code), file=self.args.stderr)
+                print(u"On the other computer, please run:", file=self.args.stderr)
+                print(u"   wormhole send --code {} <filename>".format(code), file=self.args.stderr)
+
+            else:
+                prompt = "Enter receive wormhole code: "
+                used_completion = yield input_with_completion(
+                    prompt, w.input_code(), self._reactor)
+                if not used_completion:
+                    print(
+                        " (note: you can use <Tab> to complete words)",
+                        file=self.args.stderr)
         yield w.get_code()
 
     def _show_verifier(self, verifier_bytes):
@@ -268,7 +274,7 @@ class Receiver:
     def _parse_offer(self, them_d, w):
         if "message" in them_d:
             self._handle_text(them_d, w)
-            returnValue(None)
+            return None
         # transit will be created by this point, but not connected
         if "file" in them_d:
             f = self._handle_file(them_d)
@@ -306,9 +312,13 @@ class Receiver:
                       (free, self.xfersize))
             raise TransferRejectedError()
 
+        # note the repr() here is (at least partially) to guard
+        # against malicious filenames that might play with how
+        # terminals display things. see also
+        # e.g. https://github.com/magic-wormhole/magic-wormhole/issues/476
         self._msg(u"Receiving file (%s) into: %s" %
                   (naturalsize(self.xfersize),
-                   os.path.basename(self.abs_destname)))
+                   repr(os.path.basename(self.abs_destname))))
         self._ask_permission()
         tmp_destname = self.abs_destname + ".tmp"
         return open(tmp_destname, "wb")
@@ -316,7 +326,7 @@ class Receiver:
     def _handle_directory(self, them_d):
         file_data = them_d["directory"]
         zipmode = file_data["mode"]
-        if zipmode != "zipfile/deflated":
+        if not zipmode.startswith("zipfile"):
             self._msg(u"Error: unknown directory-transfer mode '%s'" %
                       (zipmode, ))
             raise RespondError("unknown mode")
@@ -332,11 +342,14 @@ class Receiver:
 
         self._msg(u"Receiving directory (%s) into: %s/" %
                   (naturalsize(self.xfersize),
-                   os.path.basename(self.abs_destname)))
+                   repr(os.path.basename(self.abs_destname))))
         self._msg(u"%d files, %s (uncompressed)" %
                   (file_data["numfiles"], naturalsize(file_data["numbytes"])))
         self._ask_permission()
-        f = tempfile.SpooledTemporaryFile()
+        # max_size here matches the magic-number in cmd_send and will
+        # use up to 10MB of memory before putting the file on disk
+        # instead.
+        f = tempfile.SpooledTemporaryFile(max_size=10*1000*1000)
         # workaround for https://bugs.python.org/issue26175 (STF doesn't
         # fully implement IOBase abstract class), which breaks the new
         # zipfile in py3.7.0 that expects .seekable
@@ -356,12 +369,12 @@ class Receiver:
         # get confirmation from the user before writing to the local directory
         if os.path.exists(abs_destname):
             if self.args.output_file:  # overwrite is intentional
-                self._msg(u"Overwriting '%s'" % destname)
+                self._msg(u"Overwriting %s" % repr(destname))
                 if self.args.accept_file:
                     self._remove_existing(abs_destname)
             else:
                 self._msg(
-                    u"Error: refusing to overwrite existing '%s'" % destname)
+                    u"Error: refusing to overwrite existing %s" % repr(destname))
                 raise TransferRejectedError()
         return abs_destname
 
@@ -374,7 +387,7 @@ class Receiver:
     def _ask_permission(self):
         with self.args.timing.add("permission", waiting="user") as t:
             while True and not self.args.accept_file:
-                ok = six.moves.input("ok? (Y/n): ")
+                ok = input("ok? (Y/n): ")
                 if ok.lower().startswith("y") or len(ok) == 0:
                     if os.path.exists(self.abs_destname):
                         self._remove_existing(self.abs_destname)
@@ -391,7 +404,7 @@ class Receiver:
     def _establish_transit(self):
         record_pipe = yield self._transit_receiver.connect()
         self.args.timing.add("transit connected")
-        returnValue(record_pipe)
+        return record_pipe
 
     @inlineCallbacks
     def _transfer_data(self, record_pipe, f):
@@ -418,7 +431,7 @@ class Receiver:
             self._msg(u"got %d bytes, wanted %d" % (received, self.xfersize))
             raise TransferError("Connection dropped before full file received")
         assert received == self.xfersize
-        returnValue(datahash)
+        return datahash
 
     def _write_file(self, f):
         tmp_name = f.name
@@ -449,12 +462,12 @@ class Receiver:
 
         self._msg(u"Unpacking zipfile..")
         with self.args.timing.add("unpack zip"):
-            with zipfile.ZipFile(f, "r", zipfile.ZIP_DEFLATED) as zf:
+            with zipfile.ZipFile(f, "r") as zf:
                 for info in zf.infolist():
                     self._extract_file(zf, info, self.abs_destname)
 
-            self._msg(u"Received files written to %s/" % os.path.basename(
-                self.abs_destname))
+            self._msg(u"Received files written to %s/" % repr(os.path.basename(
+                self.abs_destname)))
             f.close()
 
     @inlineCallbacks
